@@ -202,7 +202,7 @@ gql_get_project_jobs = """
         statusList
       }},
       countJobInstance,
-      instances(limit: {1}){{
+      instances{1}{{
         id,
         status,
         startTime,
@@ -383,7 +383,7 @@ gql_get_project_web_apps = """
       name,
       description,
       countJobInstance,
-      instances(limit: {1}){{
+      instances{1}{{
         id,
         status,
         startTime,
@@ -494,7 +494,7 @@ gql_get_pipelines = """
         statusList
       }},
       pipelineInstanceCount,
-      instances(limit: {1}){{
+      instances{1}{{
         id,
         status,
         startTime,
@@ -575,3 +575,26 @@ gql_run_pipeline = """
     }}
   }}
 """
+
+gql_create_pipeline = """
+  mutation {{
+      createPipeline(pipeline: {{
+          name: "{0}",
+          description: "{1}",
+          projectId: "{2}",
+          jobsId: {3},
+          isScheduled: false
+      }}){{id}}
+  }}
+"""
+
+gql_get_pipeline_instance = """
+  query {{
+      pipelineInstance(id: "{0}"){{
+          id,
+          status,
+          startTime,
+          endTime
+      }}
+  }}
+"""
\ No newline at end of file
