@@ -5,6 +5,7 @@ Projects & Jobs - to interact with the manager API, see the manager subpackage)
 """
 import json
 import time
+import re
 from pathlib import Path
 
 from gql import Client
@@ -76,6 +77,35 @@ class SaagieApi:
             fetch_schema_from_transport=True
         )
 
+    @classmethod
+    def easy_connect(cls, url_saagie_platform, user, password):
+        """
+        Alternative constructor which uses the complete URL (eg: 
+        https://saagie-workspace.prod.saagie.io/projects/platform/6/) and will 
+        parse it in order to retrieve the platform URL, platform id and the 
+        realm.
+
+        Parameters
+        ----------
+        url_saagie_platform : str
+            Complete platform URL (eg: https://saagie-workspace.prod.saagie.io/projects/platform/6/)
+        user : str
+            username to login with
+        password : str
+            password to login with
+        """
+        url_regex = re.compile(
+            "(https:\/\/(\w+)-(?:\w|\.)+)\/projects\/platform\/(\d+)")
+        m = url_regex.match(url_saagie_platform)
+        if bool(m):
+            url_saagie = m.group(1)
+            realm = m.group(2)
+            id_platform = m.group(3)
+        else:
+            raise ValueError(
+                "Please use a correct URL (eg: https://saagie-workspace.prod.saagie.io/projects/platform/6/)")
+        return cls(url_saagie, id_platform, user, password, realm)
+
     # ######################################################
     # ###                    env vars                   ####
     # ######################################################
@@ -369,7 +399,7 @@ class SaagieApi:
         for repo in repositories:
             if repo['name'] == 'Saagie':
                 technologies = [techno for techno in repo['technologies']
-                                if techno['__typename'] == 'JobTechnology']
+                                if techno['__typename'] == 'JobTechnology' or (techno['__typename'] == 'SparkTechnology')]
 
         # Generate the technology graphQL string only with technologies id
         technologies = [f'{{id: "{tech["id"]}"}}' for tech in technologies]
@@ -408,7 +438,7 @@ class SaagieApi:
     # ###                      jobs                     ####
     # ######################################################
 
-    def get_project_jobs(self, project_id, instances_limit):
+    def get_project_jobs(self, project_id, instances_limit=-1):
         """List jobs in the given project with their instances.
         NB: You can only list jobs if you have at least the viewer role on the
         project
@@ -420,7 +450,7 @@ class SaagieApi:
             '/project/' (eg: the project UUID is
             '8321e13c-892a-4481-8552-5be4b6cc5df4' in
             https://saagie-workspace.prod.saagie.io/projects/platform/6/project/8321e13c-892a-4481-8552-5be4b6cc5df4/jobs)
-        instances_limit : int
+        instances_limit : int, optional
             Maximum limit of instances to fetch per job. Fetch from most recent
             to oldest
 
@@ -429,8 +459,8 @@ class SaagieApi:
         dict
             Dict of jobs information
         """
-        instances_limit = str(instances_limit)
-        query = gql(gql_get_project_jobs.format(project_id, instances_limit))
+        instances_limit_request =  f" (limit: {str(instances_limit)})" if instances_limit!=-1 else ""
+        query = gql(gql_get_project_jobs.format(project_id, instances_limit_request))
         return self.client.execute(query)
 
     def get_project_job(self, job_id):
@@ -569,7 +599,7 @@ class SaagieApi:
         query = gql(gql_edit_job.format(job))
         return self.client.execute(query)
 
-    def create_job(self, job_name, project_id, file, description='',
+    def create_job(self, job_name, project_id, file=None, description='',
                    category='Processing', technology='python',
                    runtime_version='3.6',
                    command_line='python {file} arg1 arg2', release_note='',
@@ -600,7 +630,7 @@ class SaagieApi:
             '/project' (eg: the project UUID is
             '8321e13c-892a-4481-8552-5be4b6cc5df4' in
             https://saagie-workspace.prod.saagie.io/projects/platform/6/project/8321e13c-892a-4481-8552-5be4b6cc5df4/jobs)
-        file : str
+        file : str, optional
             Local path of the file to upload
         description : str, optional
             Description of the job
@@ -648,32 +678,50 @@ class SaagieApi:
             technology_id = job_technology[0]
 
         if extra_technology != '':
-            extra_technology = extra_technology.capitalize()
             extra_tech = gql_extra_technology.format(extra_technology,
                                                      extra_technology_version)
         else:
             extra_tech = ''
 
-        with file.open(mode='rb') as f:
-            files = {
-                '1': (file.name, f),
-                'operations': (None, gql_create_job.format(job_name,
-                                                           project_id,
-                                                           description,
-                                                           category,
-                                                           technology_id,
-                                                           runtime_version,
-                                                           command_line,
-                                                           release_note,
-                                                           extra_tech)),
-                'map': (None, '{ "1": ["variables.file"] }'),
-            }
-
-            url = self.url_saagie + self.suffix_api + 'platform/'
-            url += str(self.id_platform) + "/graphql"
+        url = self.url_saagie + self.suffix_api + 'platform/'
+        url += str(self.id_platform) + "/graphql"
+        
+        if file:
+            file = Path(file)
+            #logging.debug("Creating jobs with archive ...")
+            with file.open(mode='rb') as f:
+                files = {
+                    '1': (file.name, f),
+                    'operations': (None, gql_create_job.format(job_name,
+                                                            project_id,
+                                                            description,
+                                                            category,
+                                                            technology_id,
+                                                            runtime_version,
+                                                            command_line,
+                                                            release_note,
+                                                            extra_tech)),
+                    'map': (None, '{ "1": ["variables.file"] }'),
+                }
+                response = requests.post(url,
+                                        files=files,
+                                        auth=self.auth,
+                                        verify=False)
+        else:
+            payload_str = gql_create_job.format(job_name,
+                                        project_id,
+                                        description,
+                                        category,
+                                        technology_id,
+                                        runtime_version,
+                                        command_line,
+                                        release_note,
+                                        extra_tech)
+            payload = json.loads(payload_str)
             response = requests.post(url,
-                                     files=files,
-                                     auth=self.auth)
+                                        json=payload,
+                                        auth=self.auth,
+                                        verify=False)
 
         if response:
             return json.loads(response.content)
@@ -706,7 +754,7 @@ class SaagieApi:
 
     # Difference between app and webapp in current graphQL API ?
     # Web App = Docker jobs ?
-    def get_project_web_apps(self, project_id, instances_limit):
+    def get_project_web_apps(self, project_id, instances_limit=-1):
         """List webApps of project with their instances.
         NB: You can only list webApps if you have at least the viewer role on
         the project.
@@ -718,7 +766,7 @@ class SaagieApi:
             '/project' (eg: the project UUID is
             '8321e13c-892a-4481-8552-5be4b6cc5df4' in
             https://saagie-workspace.prod.saagie.io/projects/platform/6/project/8321e13c-892a-4481-8552-5be4b6cc5df4/jobs)
-        instances_limit : int
+        instances_limit : int, optional
             Maximum limit of instances to fetch per webapp. Fetch from most
             recent to oldest
 
@@ -727,9 +775,22 @@ class SaagieApi:
         dict
             Dict of webApp information
         """
+        regex_error_missing_technology = "io\.saagie\.projectsandjobs\.domain\.exception\.NonExistingTechnologyException: Technology \S{8}-\S{4}-\S{4}-\S{4}-\S{12} does not exist"
+        instances_limit_request =  f" (limit: {str(instances_limit)})" if instances_limit!=-1 else ""
+        
         query = gql(gql_get_project_web_apps.format(project_id,
-                                                    instances_limit))
-        return self.client.execute(query)
+                                                    instances_limit_request))
+        result = self.client._get_result(query) 
+        
+        if result.errors:
+            # Matching errors with error missing technology message
+            errors_matching = [re.fullmatch(regex_error_missing_technology, e['message']) for e in result.errors]
+            if None in errors_matching:
+                raise Exception(str(result.errors[errors_matching.index(None)]))
+            else:
+                return result.data
+        else:
+            return result.data
 
     def get_project_web_app(self, web_app_id):
         """Get webApp with given UUID or null if it doesn't exist.
@@ -768,7 +829,7 @@ class SaagieApi:
     # ######################################################
     # ###                   pipelines                   ####
     # ######################################################
-    def get_project_pipelines(self, project_id, instances_limit):
+    def get_project_pipelines(self, project_id, instances_limit=-1):
         """List pipelines of project with their instances.
 
         Parameters
@@ -778,7 +839,7 @@ class SaagieApi:
             '/project' (eg: the project UUID is
             '8321e13c-892a-4481-8552-5be4b6cc5df4' in
             https://saagie-workspace.prod.saagie.io/projects/platform/6/project/8321e13c-892a-4481-8552-5be4b6cc5df4/jobs)
-        instances_limit : int
+        instances_limit : int, optional
             Maximum limit of instances to fetch per pipelines. Fetch from most
             recent to oldest
 
@@ -787,7 +848,8 @@ class SaagieApi:
         Dict
             Dict of pipelines information
         """
-        query = gql(gql_get_pipelines.format(project_id, instances_limit))
+        instances_limit_request =  f" (limit: {str(instances_limit)})" if instances_limit!=-1 else ""
+        query = gql(gql_get_pipelines.format(project_id, instances_limit_request))
         return self.client.execute(query)
 
     def get_project_pipeline(self, pipeline_id):
@@ -921,3 +983,51 @@ class SaagieApi:
                 .get("status")
             print('Current state : ' + state)
         return state
+
+    def create_pipeline(self, name, project_id, jobs_id, description=""):
+        """
+        Create a pipeline in a given project
+
+        Parameters
+        ----------
+        name : str
+            Name of the pipeline. Must not already exist in the project
+        project_id : str
+            UUID of your project. Can be found in the project URL after the
+            '/project' (eg: the project UUID is
+            '8321e13c-892a-4481-8552-5be4b6cc5df4' in
+            https://saagie-workspace.prod.saagie.io/projects/platform/6/project/8321e13c-892a-4481-8552-5be4b6cc5df4/jobs)
+        jobs_id : List
+            Ordered list of job's id (example : ["id1", "id2", "id3"] will result in the following pipeline id1 -> id2 -> id3)
+        description : str, optional
+            Description of the pipeline
+
+        Returns
+        -------
+        dict
+            Dict of job information
+        """
+        query = gql(gql_create_pipeline.format(
+            name,
+            description,
+            project_id,
+            f"""["{'", "'.join(jobs_id)}"]"""
+        ))
+        return self.client.execute(query)
+
+    def get_pipeline_instance(self, pipeline_instance_id):
+        """
+        Get the information of a given pipeline instance id
+
+        Parameters
+        ----------
+        pipeline_instance_id : str
+            Pipeline instance id
+
+        Returns
+        -------
+        dict
+            Dict of job information
+        """
+        query = gql(gql_get_pipeline_instance.format(pipeline_instance_id))
+        return self.client.execute(query)
